
import 'package:coders/scanner/token.dart';

enum NodeType {
  Numeric,
  Value,
  Keyword,
  Identifier,
  Punctuator,
  AssignOperator,
  UnaryOperator,
  UpdateOperator,
  DataType,
  BlockStatement,
  Parameter,
  TypeDeclaration,
  TableDeclaration,
  TableMemberDeclaration,
  StructMemberDeclaration,
  StructDeclaration,
  FuncDeclaration,
  VarDeclaration,
  Expression,
  UpdateExpression,
  UnaryExpression
}


class INode {

  /// node type
  NodeType nodeType;

  /// next node
  INode next;

  /// prev node
  INode prev;

}

class Node implements INode {

  @override
  INode next;

  @override
  INode prev;

  @override
  NodeType nodeType;

  Token token;

  String get value => token.value;

  Node(this.nodeType);
}

class Numeric extends Node {
  Numeric(): super(NodeType.Numeric);
}

class Keyword extends Node {
  Keyword(): super(NodeType.Keyword);
}

class Punctuator extends Node {
  Punctuator(): super(NodeType.Punctuator);
}

class Identifier extends Node {
  Identifier(): super(NodeType.Identifier);
}

class Value extends Node {
  Value() : super(NodeType.Value);
}


class AssignOperator extends Node {
  AssignOperator(): super(NodeType.AssignOperator);
}

class UnaryOperator extends Node {
  UnaryOperator(): super(NodeType.UnaryOperator);
}

class UpdateOperator extends Node {
  UpdateOperator(): super(NodeType.UpdateOperator);
}