import 'ast_node.dart';
import 'statement.dart';


class StructMemberDeclaration extends Statement {
  StructMemberDeclaration() : super(NodeType.StructMemberDeclaration);

  Identifier name;

  DataType dataType;

  Punctuator semicolon;
}

class TableMemberDeclaration extends Statement {
  TableMemberDeclaration() : super(NodeType.TableMemberDeclaration);

  Identifier name;

  DataType dataType;

  Punctuator equal;

  Value defaultValue;

  Keyword nullable;

  Keyword auto;

  Punctuator semicolon;
}

class DataType extends Statement {
  DataType() : super(NodeType.DataType);

  Identifier name;

  Punctuator lbraket;

  Numeric length;

  Punctuator rbraket;
}

class Parameter extends Statement {
  Parameter() : super(NodeType.Parameter);

  Keyword inout;

  Identifier name;

  DataType dataType;
}

class BlockStatement extends Statement {
  BlockStatement() : super(NodeType.BlockStatement);

  Punctuator lbrace;

  var stmt = <INode>[];

  Punctuator rbrace;
}

class VarDeclaration extends Statement {
  VarDeclaration() : super(NodeType.VarDeclaration);

  Keyword varKeyword;

  Identifier name;

  DataType dataType;

  Punctuator equal;

  Value defaultValue;

  Punctuator semicolon;
}

class FuncDeclaration extends Statement {
  FuncDeclaration() : super(NodeType.FuncDeclaration);

  Keyword func;

  Identifier name;

  Punctuator lbraket;

  var parameters = <Parameter>[];

  Punctuator rbraket;

  DataType returnType;

  BlockStatement block;

}

class StructDeclaration extends Statement {
  StructDeclaration() : super(NodeType.StructDeclaration);

  Keyword struct;

  Identifier name;

  Punctuator lbrace;

  var memberDecls = <StructMemberDeclaration>[];

  Punctuator rbrace;
}

class TypeDeclaration extends Statement {
  TypeDeclaration() : super(NodeType.TypeDeclaration);

  Keyword type;

  Identifier name;

  DataType dataType;

  Punctuator semicolon;
}

class TableDeclaration extends Statement {
  TableDeclaration() : super(NodeType.TypeDeclaration);

  Keyword table;

  Identifier name;

  Punctuator lbrace;

  var memberDecls = <TableMemberDeclaration>[];

  Punctuator rbrace;
}


class Expression extends Node {
  Expression() : super(NodeType.Expression);

  Node left;

  AssignOperator assignOperator;

  Expression right;
}

class UpdateExpression extends Node {
  UpdateExpression() : super(NodeType.UpdateExpression);

  UpdateOperator operator;
  Identifier name;
}

class UnaryExpression extends Node {
  UnaryExpression() : super(NodeType.UnaryExpression);

  UnaryOperator operator;
  Identifier name;
}
