
import 'package:coders/scanner/location.dart';
import 'package:coders/scanner/token.dart';


class BaseException implements Exception {

  int line;
  int column;
  String value;

  BaseException(Token token) {
    line = token.location.begLine + 1;
    column = token.location.begColumn + 1;
    value = token.value;
  }

  BaseException.location(Location location, String value) {
    line = location.begLine + 1;
    column = location.begColumn + 1;
    this.value = value;
  }


  @override
  String toString() {
    return 'exception (${line}, ${column}) - ${value}';
  }
}

class UnexpectedToken extends BaseException {

  UnexpectedToken(Token token) : super(token);

  UnexpectedToken.location(Location location, String value): super.location(location, value);

  @override
  String toString() {
    return 'unexpected token (${line}, ${column}) - ${value}';
  }
}


class DuplicateToken extends BaseException {
  
  DuplicateToken(Token token) : super(token);

  @override
  String toString() {
    return 'duplicated token (${line}, ${column}) - ${value}';
  }
}
