import 'package:coders/exception/exceptions.dart';
import 'package:coders/scanner/scanner.dart';
import 'package:coders/scanner/token.dart';
import 'package:test/test.dart';

void main() {



  /**
   func hello(in name varchar(32)) integer {
     return 0;
   }

   struct User {
     userId varchar(32);
     name varchar(32);
   }

   @resultset(User, List<User>)
   proc hello(in _name varchar(32)) integer {

     var _userId varchar(32) = '';

     select userId into _userId
     from user
     where name = _name;

     select userId, name
     from user
     where name = _userId;



     return 0;
   }
   */

  test('token-assign', () {
    var source = '*= /= %= += -=';
    var scanner = Scanner(source);
    var token = scanner.scan();
    while(token.type != TokenType.Eof) {
      assert(token.type == TokenType.Punctuator);
      token = scanner.scan();
    }
  });

  test('token-string', () {
    var source = "'aaa'";

    var scanner = Scanner(source);
    var token = scanner.scan();
    assert(token.type == TokenType.StringLiteral);
    assert(token.value == "'aaa'");

    source = "'aaa\\'bbb'";
    scanner = Scanner(source);
    token = scanner.scan();
    assert(token.type == TokenType.StringLiteral);
    assert(token.value == "'aaa\\'bbb'");
    
  });

  test('token-numeric', () {
    var source = '''1234''';
    var scanner = Scanner(source);
    var token = scanner.scan();
    assert(token.type == TokenType.NumericLiteral);
    assert(token.value == '1234');
    assert(token.location.endColumn == 3);

    source = '''.1234''';
    scanner = Scanner(source);
    token = scanner.scan();
    assert(token.type == TokenType.NumericLiteral);
    assert(token.value == '.1234');
    assert(token.location.endColumn == 4);

    source = '''0x12''';
    scanner = Scanner(source);
    token = scanner.scan();
    assert(token.type == TokenType.NumericLiteral);
    assert(token.value == '0x12');
    assert(token.location.endColumn == 3);

    source = '''01234''';
    scanner = Scanner(source);
    token = scanner.scan();
    assert(token.type == TokenType.NumericLiteral);
    assert(token.value == '01234');
    assert(token.location.endColumn == 4);

    source = '''01234''';
    scanner = Scanner(source);
    token = scanner.scan();
    assert(token.type == TokenType.NumericLiteral);
    assert(token.value == '01234');
    assert(token.location.endColumn == 4);

  });

  test('token-keywords-type', () {
    final source = 'type UserID varchar(32);';
    final scanner = Scanner(source);
    var token = scanner.scan();
    assert(token.type == TokenType.Keyword);
    token = scanner.scan();
    assert(token.type == TokenType.Identifier);
    token = scanner.scan();
    assert(token.type == TokenType.Keyword);
    token = scanner.scan();
    assert(token.type == TokenType.Punctuator);
    token = scanner.scan();
    assert(token.type == TokenType.NumericLiteral);
    token = scanner.scan();
    assert(token.type == TokenType.Punctuator);
    token = scanner.scan();
    assert(token.type == TokenType.Punctuator);
    token = scanner.scan();
    assert(token.type == TokenType.Eof);
  });

  test('token-keywords-1', () {
    final source = '''type''';
    final scanner = Scanner(source);
    var token = scanner.scan();
    assert(token.type == TokenType.Keyword);
    assert(token.value == 'type');
    assert(token.location.begColumn == 0);
    assert(token.location.endColumn == 3);
    assert(token.location.begLine == 0);
    assert(token.location.endLine == 0);
  });

  test('token-keywords-2', () {
    final source = '''type var''';
    final scanner = Scanner(source);
    var token = scanner.scan();
    assert(token.type == TokenType.Keyword);
    assert(token.value == 'type');
    assert(token.location.begColumn == 0);
    assert(token.location.endColumn == 3);
    assert(token.location.begLine == 0);
    assert(token.location.endLine == 0);

    token = scanner.scan();
    assert(token.type == TokenType.Keyword);
    assert(token.value == 'var');
    assert(token.location.begLine == 0);
    assert(token.location.begColumn == 5);
    assert(token.location.endLine == 0);
    assert(token.location.endColumn == 7);
  });


  test('token - comment', () {
    final source = '''
// This is a comment.
''';

    final scanner = Scanner(source);
    final token = scanner.scan();
    assert(token.type == TokenType.Eof);
    assert(token.value == '');
    assert(token.comments.length == 1);
    assert(token.comments[0].value == '// This is a comment.');
    assert(token.comments[0].location.begLine == 0);
    assert(token.comments[0].location.begColumn == 0);
    assert(token.comments[0].location.endLine == 0);
    assert(token.comments[0].location.endColumn == 21);
  });

  test('token - comment', () {
    final source = '''/*\n  This is block comment \n*/''';

    final scanner = Scanner(source);
    final token = scanner.scan();
    assert(token.type == TokenType.Eof);
    assert(token.value == '');
    assert(token.comments.length == 1);
    assert(token.comments[0].value == '''/*\n  This is block comment \n*/''');
    assert(token.comments[0].location.begColumn == 0);
    assert(token.comments[0].location.endColumn == 2);
    assert(token.comments[0].location.begLine == 0);
    assert(token.comments[0].location.endLine == 2);

  });
}
